;|===========================================================================|
;|                               ADDITION                                    |
;| This routine adds the numbers A and B in floating point arithmetic. Each  |
;| number A and B are stored into 5 bytes of ram space. The result is saved  |
;| into 6 bytes of ram space.                                                |
;|===========================================================================|

;* This section shifts the number A or B according to decimal point position 
;* of A and B.

additn   ld a,pointa         ; Read the point position of number A.
         sub a,pointb
         jrnz addit1         ; Comparison of decimal point position.
         ld a,pointa         ; No shift of the number A or B.
         ld pointr,a         ; Pointa = pointb = pointr.
         jp addit6
addit1 	 jrnc addit2
         jp addit3
addit2	  ld result,a         ; Pointa > pointb.
         ldi a,05h
         sub a,result
         ld byte_num,a       ; Load the number of bytes to be shifted.
         ld a,result
         addi a,lsb_a
         ld x,a              ; Load the first byte address to be shifted.
         ldi y,lsb_a
         ld a,pointb
         ld pointr,a         ; Pointr = pointb.
         jp addit4
addit3 	 ld a,pointb         ; Pointa < pointb.
         sub a,pointa
         ld result,a
         ldi a,05h
         sub a,result
         ld byte_num,a       ; Load the number of bytes to be shifted.
         ld a,result
         addi a,lsb_b 
         ld x,a              ; Load the first byte address to be shifted.
         ldi y,lsb_b
         ld a,pointa
         ld pointr,a         ; Pointr = pointa.
addit4   ld a,(x)            ; Read the byte.
         ld (y),a
         inc y
         inc x
         dec byte_num
         jrnz addit4         ; Test if all bytes have been shifted ?
addit5   clr a
         ld (y),a            ; Complete others byte to zero.
         inc y
         ld a,y
         cp a,x
         jrnz addit5

;* This section adds number A to B

addit6   ldi byte_num,05h    ; Load the number of Bytes to be added.
         clr overflow        ; Carry = 0.
         ldi addr_r,lsb_r    ; Save the lsb address of the resultant addition.
         ldi addr_a,lsb_a    ; Save the lsb address of the number A.
         ldi x,lsb_a         ; Load the lsb address of the number A.
         ldi y,lsb_b         ; Load the lsb address of the number B.
additn7  ld a,(x) 
         add a,(y)
         add a,overflow      ; Byte A + byte B + carry.
         ld result,a         ; Save the result in result register.
         subi a,0ah          ; Test if the result < 10.
         jrnc additn8
         clr overflow        ; No carry, then overflow = 0.
         jp additn9
additn8  ld result,a         ; Save the converting result in result register.
         ldi overflow,01h    ; If carry, then overflow = 1.
additn9  ld a,addr_r
         ld x,a              ; Load the address of the resultant addition byte.
         ld a,result
         ld (x),a            ; Store in the resultant addition byte.
         inc addr_r
         inc y
         inc addr_a
         ld a,addr_a
         ld x,a
         dec byte_num
         jrz additn10        ; Test for the end of the addition ?
         jp additn7
additn10 ld a,addr_r
         ld x,a
         ld a,overflow
         ld (x),a            ; Store 1 or 0 in the sixth byte according to
                             ; the overflow value
end_addi ret
