;*****************************************************************************
;*                                                                           *
;*                               CALCULATOR                                  *
;*                                                                           *
;*****************************************************************************

;|===========================================================================|
;| Assembly method without linker : ast6 -l -s -x -m -e calcul.asm           |
;|===========================================================================|

;|===========================================================================|
;|                          Assembly directives                              |
;|===========================================================================|

	 .title	"CALCULATOR"      ; This directive defines a new title to be
				  ; in the page header of the output listing.
         .vers "st6215"
         .romsize 2
         .w_on                    ; This directive must be included if in data
                                  ; space a "window" exists to access to data
                                  ; defined in program space.
	 .pp_on                   ; This directive is to be included when the
				  ; program space is segmented in 2 K page.
				  ; This directive allows the use of .SECTION
				  ; directive.

;*****************************************************************************
;*                               Data space                                  *
;*****************************************************************************

;|===========================================================================|
;|                             Data registers                                |
;|===========================================================================|

	 .input "6215_reg.asm"

;|===========================================================================|
;|                                data ram                                   |
;|===========================================================================|

kdep_cnt .def 84h                 ; The depressed key counter is increased
                                  ; in the timer interrupt routine.
krel_cnt .def 85h                 ; The realesed key counter is increased in
                                  ; the timer interrupt routine.
key_val  .def 86h                 ; Saves the depressed key if it is present
                                  ; for more than 40 ms.
key_sav  .def 87h                 ; Saves the value of a depressed key in
                                  ; order to compare that the keypad entry is
                                  ; always the same during the keypad
                                  ; processing time.
overflow .def 88h                 ; Saves the carry of each operation.
result   .def 89h                 ; Saves the result of each operation.
addr_r   .def 8ah                 ; Saves the byte address of the resultant
                                  ; addition.
addr_a   .def 8bh                 ; Saves the byte address of number A.
byte_num .def 8ch                 ; The byte number has to be shifted, the
                                  ; byte number has to be added and the byte
                                  ; number has to be multiplied.
pointa   .def 8dh                 ; The decimal point position of the number A.
pointb   .def 8eh                 ; The decimal point position of the number B.
pointr   .def 8fh                 ; The decimal point position of the resultant
                                  ; addition.

key_flag .def 90h
;----------------
k_dep    .equ   0                 ; Indicates that a key has been depressed
                                  ; during the keypad scanning.
k_sav    .equ   1                 ; Indicates that a depressed key has been
                                  ; saved in key_sav register.
k_on     .equ   2                 ; Allows to process a key.

rg_keypr .def 91h                 ; key code after  key processing routine.
save_a   .def 92h                 ; Save the accumulator in interrupt routine.

ope_flag .def 93h
;----------------
mult_val .equ   1                 ; Set if the result of the multiplication
                                  ; is valided.
k_fnd    .equ   2                 ; Set when the key processing routine find
				  ; the key code.

buf0     .def 0a0h                ;\
buf1     .def 0a1h                ; |
buf2     .def 0a2h                ; | number A
buf3     .def 0a3h                ; |
buf4     .def 0a4h                ;/

buf5     .def 0a5h                ;\
buf6     .def 0a6h                ; |
buf7     .def 0a7h                ; | number B
buf8     .def 0a8h                ; |
buf9     .def 0a9h                ;/

; 6 bytes for the resultant addition ( from 0aah to 0afh )
; 16 bytes for the resultant multiplication ( from 0aah to 0b9h )

;|===========================================================================|
;|                            constants definition                           |
;|===========================================================================|

lsb_a    .equ 0a0h                ; The least significant byte address of the
                                  ; number A.
msb_a    .equ 0a4h
lsb_b    .equ 0a5h                ; The least significant byte address of the
                                  ; number B.
msb_b    .equ 0a9h                ; The most significant byte address of the
                                  ; number B.
lsb_r    .equ 0aah                ; The least significant byte address of the
                                  ; resultant addition.
msb_r    .equ 0bah                ; The most significant byte address of the
                                  ; resultant multiplication.
pointr0  .equ 0b1h                ; The decimal point address of the resultant
                                  ; multiplication.
tempo_40 .equ 04h                 ; The value of tempo_40 is equal to 40 ms
                                  ; divided by the interrupt time base.
tempo_60 .equ 06h                 ; The value of tempo_60 is equal to 60 ms
                                  ; divided by the interrupt time base.

; bits timer1
;------------
tmz      .equ 7                   ; This bit must be cleared by user software
                                  ; before to start with a new count.

;|===========================================================================|
;|                               data rom                                    |
;|===========================================================================|

;|---------------------------------------------------------------------------|
;| The effective address of the byte to be read as a data in the ROM memory  |
;| is obtained by the concatenation of the 6 less significant bits of the    |
;| address in the data space and the content of the drwr register.           |
;|---------------------------------------------------------------------------|

	 .section 1               ; This directive provides a logical
				  ; partionning of the program space. 
  
	 .org 80h

	 .block  64-$%64          ; Force page boundary.

key_tab	 .byte 0eeh               ; 0
	 .byte 0deh               ; 1
	 .byte 0beh               ; 2
	 .byte 07eh               ; 3
	 .byte 0edh               ; 4
	 .byte 0ddh               ; 5
	 .byte 0bdh               ; 6
	 .byte 07dh               ; 7
	 .byte 0ebh               ; 8
	 .byte 0dbh               ; 9
	 .byte 0b7h               ; +
	 .byte 077h               ; x
	 .byte 0e7h               ; RAZ
	 .byte 0d7h               ; .

;*****************************************************************************
;*                             PROGRAM SPACE                                 *
;*****************************************************************************

;|===========================================================================|
;|                             main program                                  |
;|===========================================================================|

reset    ldi wdr,0feh             ; Load the watchdog.
	 ldi ddra,00001111b 
	 ldi ora,00000000b
	 ldi dra,00001111b        ; The input to port A is configured with
				  ; a pull_up resistor for each of the 4 MSB's
				  ; and open drain output for the 4 LSB's.
	 ldi x,084h
inram    clr a                    ;\ 
	 ld (x),a                 ; |
	 inc x                    ; | Clear ram from 084h to 0bfh.
	 ld a,x                   ; |
	 cpi a,0c0h               ; |
	 jrc inram                ;/
	 ldi drwr,key_tab.w       ; The data rom window register is initialzed.
				  ; by the notation key_tab.w.
	 ldi ior,00010000b        ; Enable the interrupts.
       	 reti

	 ldi wdr,0feh             ; Load the watchdog.
	 ldi tcr,11010000b        ; 12 x 16 x (0d0h) x 1/4 Mhz = 10 ms
	 ldi tscr,01101100b       ; Start the timer. 

cal0	 ldi buf0,00h             ;\
	 ldi buf1,01h             ; |
	 ldi buf2,05h             ; | number A = 24,510
	 ldi buf3,04h             ; |
	 ldi buf4,02h             ; |
	 ldi pointa,03h           ;/

       	 ldi buf5,01h             ;\
       	 ldi buf6,03h             ; |
       	 ldi buf7,06h             ; | number B = 926,31
       	 ldi buf8,02h             ; |
       	 ldi buf9,09h             ; |
       	 ldi pointb,02h           ;/

cal1     wait                     ; Wait for interrupt.
       	 ld a,key_val             ; Read the key.
       	 jrz cal1                 ; Key ?
       	 call keypr               ; Key processing routine.
       	 jrr k_fnd,ope_flag,cal1  ; Test if the key processing routine find
                                  ; the key code.
       	 ld a,rg_keypr
       	 cpi a,0ah                ; Key " + " ?
       	 jrnz cal2
       	 call additn              ; Call the addition subroutine. 
	 jp cal0
cal2     cpi a,0bh                ; Key " x " ?
       	 jrz cal3
       	 jp cal1
cal3     call multicn             ; Call the multiplication subroutine.
	 jp cal0

;|===========================================================================|
;|                              subroutines                                  |
;|===========================================================================|

;* The .INPUT directive causes the source statememt(s) to be read from the 
;* specified file. 

	 .input "scan_key.asm"


       	 .input "additn.asm"


       	 .input "multipl1.asm"

;|---------------------------------------------------------------------------|
;|                             key processing                                |
;|---------------------------------------------------------------------------|

keypr    ldi y,key_tab.d          ; Load the first address of the key table.
	 ldi rg_keypr,00h         ; Initialize rg_keypr.
keypr1   ld a,key_val
       	 cp a,(y)
       	 jrz keypr2               ; Test if the key is found.
       	 inc rg_keypr             ; Increase rg_keypr.
       	 inc y                    ; Increase key table address.
	 ldi a,0eh                ; 14 bytes in key table.
	 addi a,key_tab.d
	 cp a,y                   ; Test the end of key table.
       	 jrnz keypr1
       	 res k_fnd,ope_flag
       	 jp keypr3
keypr2   set k_fnd,ope_flag
keypr3   clr key_val
       	 ret

;|===========================================================================|
;|                         interrupt routines                                |
;|===========================================================================|

;|---------------------------------------------------------------------------|
;|                          timer interrupt                                  |
;|---------------------------------------------------------------------------|

it_timer ldi wdr,0feh             ; Reload the watchdog.
	 ldi tcr,11010000b        ; reload the counter.
	 res tmz,tscr             ; Restart the timer.
	 inc kdep_cnt             ; Increase depressed key counter.
	 inc krel_cnt             ; Increase released key counter.
	 ld save_a,a              ; Save the accumulator.
	 call scan_key            ; Call the scanning keypad subroutine.
	 ld a,save_a              ; Restore the accumulator.
	 reti

;*****************************************************************************
;*                    interrupt and restart vectors                          *
;*****************************************************************************

	 .section 32

	 .org 00h

int4     nop
	 reti
int3     jp it_timer
int2     nop
       	 reti
int1     nop
       	 reti

	 .org 0ch 

nmi      nop
       	 reti
res      jp reset
