;*****************************************************************************
;*                                                                           *
;*                              ST6 EXAMPLE 2                                *
;*                                                                           *
;*****************************************************************************

;|===========================================================================|
;| Assembly method without linker : ast6 -l -s -x -m -e example2.asm         |
;|===========================================================================|

;|===========================================================================|
;|                          Assembly directives                              |
;|===========================================================================|

	 .title	"ST6 EXAMPLE 2"   ; This directive defines a new title to be
				  ; in the page header of the output listing.
	 .vers "st6215"
	 .romsize 2
	 .w_on                    ; This directive must be included if in data
				  ; space a "window" exists to access to data
				  ; defined in program space.
	 .pp_on                   ; This directive is to be included when the
				  ; program space is segmented in 2K page.
				  ; This directive allows the use of .SECTION
				  ; directive.

;*****************************************************************************
;*                              data space                                   *
;*****************************************************************************

;|===========================================================================|
;|                           data registers                                  |
;|===========================================================================|

         .input "6215_reg.asm"

;|===========================================================================|
;|                              data ram                                     |
;|===========================================================================|

display  .def 84h                 ; Variable display value.
ticks    .def 85h                 ; Counts timer interrupts.

;|===========================================================================|
;|                            constants definition                           |
;|===========================================================================|

count    .equ 0fah
watchtim .equ 0feh
numticks .equ 0ah

;bit timer
;---------
tmz      .equ 7                   ; This bit must be cleared by user software
				  ; before to start with a new count.

;|===========================================================================|
;|                                  data rom                                 |
;|===========================================================================|

;|---------------------------------------------------------------------------|
;| The effective address of the byte to be read as a data in the ROM memory  |
;| is obtained by the concatenation of the 6 less significant bits of the    |
;| address in the data space and the content of the drwr register.           |
;|---------------------------------------------------------------------------|

	 .section 1               ; This directive provides a logical
				  ; partionning of the program space. 
	
	 .org 80h

	 .block  64-$%64           ; Force page boundary.

pattern  .byte 0feh,0fdh,0fbh,0f7h,0efh,0dfh,0bfh,7fh
	 .byte 7fh,0bfh,0dfh,0efh,0f7h,0fbh,0fdh,0feh
	 .byte 0fch,0f9h,0f3h,0e7h,0cfh,9fh,3fh,7fh
	 .byte 7fh,3fh,9fh,0cfh,0e7h,0f3h,0f9h,0fch
	 .byte 0ffh,00h,0ffh,00h,0ffh,00h,0ffh,00h
	 .byte 0ffh,00h,0ffh,00h,0ffh,00h,0ffh,00h

;*****************************************************************************
;*                             PROGRAM SPACE                                 *
;*****************************************************************************

;|===========================================================================|
;|                             main program                                  |
;|===========================================================================|

reset    ldi wdr,watchtim         ; Load the watchdog.

	 ldi ddra,11111111b 
	 ldi ora,00000000b
	 ldi dra,00000000b        ; The port A is configured in open drain 
				  ; output.
	 ldi ddrb,11111111b 
	 ldi orb,00000000b
	 ldi drb,00000000b        ; The port B is configured in open drain
				  ; output.
	 ldi drwr,pattern.w       ; The data rom window register is initialzed.
				  ; by the notation pattern.w.
	 ldi ior,10h              ; Enables  all interrupts.
	 reti 

	 ldi tcr,count		  ; 12 x 4 x (0fah) x 1/4 Mhz = 3 ms
	 ldi tscr,01101100b       ; Start the timer. 

	 clr display
	 clr ticks
	 ldi x,pattern.d          ; X points to first pattern.
loop     wait                     ; Wait for interrupt.
	 ld a,ticks
	 cpi a,numticks           ; Check number of ticks.
	 jrnz loop
	 call update
	 clr ticks
	 jp loop                  ; Continue.

;|===========================================================================|
;|                              subroutines                                  |
;|===========================================================================|

;|---------------------------------------------------------------------------|
;|                                  update                                   |
;|---------------------------------------------------------------------------|

update   ld a,(x)
	 ld drb,a                 ; Display element.
	 inc x                    ; Next element of pattern.
	 ldi a,30h
	 addi a,pattern.d
	 cp a,x
	 jrnz next
	 ldi x,pattern.d          ; X points the first pattern and the
				  ; first element.
next	 dec display              ; Decrement display value.
	 ld a,display
	 ld dra,a                 ; Load data reg with display
	 ret				

;|===========================================================================|
;|                         interrupt routines                                |
;|===========================================================================|

;|---------------------------------------------------------------------------|
;|                          timer interrupt                                  |
;|---------------------------------------------------------------------------|

it_timer ldi wdr,watchtim         ; Reload the watchdog.
	 ldi tcr,count            ; Reload the counter.
	 res tmz,tscr             ; Restart the timer.
	 inc ticks
	 reti

;*****************************************************************************
;*                     Restart and interrupt Vectors                         *
;*****************************************************************************

         .section 32

	 .org 00h

adc      nop
	 reti
timer    jp it_timer
int2     nop
         reti
int1     nop
	 reti
	 
	 .org 0ch

nmi      nop
	 reti
res      jp reset
