;*****************************************************************************
;*                                                                           *
;*                  ST6 ANALOG TO DIGITAL CONVERTER                          *
;*                                                                           *
;*****************************************************************************
;*****************************************************************************
;* This program reads an analogic value on port A bit 0 PA0 and output	     *	
;* the value read on the bits of port B PB0->PB7.			     *
;*****************************************************************************

;|===========================================================================|
;| Assembly method without linker : ast6 -l -s -x -m -e example1.asm         |
;|===========================================================================|

;|===========================================================================|
;|                          Assembly directives                              |
;|===========================================================================|

	 .title	"ST6 ADC"   ; This directive defines a new title to be
				  ; in the page header of the output listing.
	 .vers "st6215"
	 .romsize 2

;*****************************************************************************
;*                              data space                                   *
;*****************************************************************************

;|===========================================================================|
;|                           data registers                                  |
;|===========================================================================|

         .input "6215_reg.asm"

;|===========================================================================|
;|                            constants definition                           |
;|===========================================================================|

count	 .equ 0fah
watchtim .equ 0feh

;bit timer
;---------
tmz      .equ 7                   ; This bit must be cleared by user software
				  ; before to start with a new count.
dout     .equ 4                   ; Data sent to the timer output  when tms
				  ; is set hight.

;*****************************************************************************
;*                             PROGRAM SPACE                                 *
;*****************************************************************************

;|===========================================================================|
;|                             main program                                  |
;|===========================================================================|

	 .org 880h
;**************************************************************************
;*  CONFIGURATION of port A (DDRA = 0,DRA = 01,ORA = 01) => PA0 analog input
;**************************************************************************
reset	 ldi ddra,0               ; Load the data direction register port A
	 ldi dra,01		  ; Load data register port A 
	 ldi ora,01		  ; Load option register port A
;**************************************************************************
;*  Configuration of port B in all output
;**************************************************************************
	 ldi ddrb,0FFh
	 ldi orb,00
	 ldi drb,0FFh
	 ldi drb,00
;**************************************************************************
;*  Configuration of the ADC
;**************************************************************************
	 ldi adcr,0B0h		  ; enable ADC interrupt and conversion
;**************************************************************************
;*  Enable the interrupts => IOR = 10h && reti (quit NMI mode after reset)
;**************************************************************************
	 ldi ior,10h              ; Enables all interrupts.
	 reti  			  ; Quit NMI mode after reset

loop	 wait			  ; wait for a timer interrupt
	 jp loop
;*****************************************************************************
;* interrupt routine for ADC
;*****************************************************************************
it_tim	res 6,tscr		  ; reset the TMZ bit
	ld a,adr
	ld drb,a
	reti

;*****************************************************************************
;*                     Restart and interrupt Vectors                         *
;*****************************************************************************

	 .org 0ff0h

adc      jp it_tim
timer    nop
	 reti
portb    nop
         reti
porta    nop
	 reti
	 nop
	 nop
	 nop
	 nop
nmi      nop
	 reti
res      jp reset
