;*****************************************************************************
;*                                                                           *
;*                             ST6 EXAMPLE 1                                 *
;*                                                                           *
;*****************************************************************************

;|===========================================================================|
;| Assembly method without linker : ast6 -l -s -x -m -e example1.asm         |
;|===========================================================================|

;|===========================================================================|
;|                          Assembly directives                              |
;|===========================================================================|

	 .title	"ST6 EXAMPLE 1"   ; This directive defines a new title to be
				  ; in the page header of the output listing.
	 .vers "st6215"
	 .romsize 2

;*****************************************************************************
;*                              data space                                   *
;*****************************************************************************

;|===========================================================================|
;|                           data registers                                  |
;|===========================================================================|

         .input "6215_reg.asm"

;|===========================================================================|
;|                            constants definition                           |
;|===========================================================================|

count	 .equ 0fah
watchtim .equ 0feh

;bit timer
;---------
tmz      .equ 7                   ; This bit must be cleared by user software
				  ; before to start with a new count.
dout     .equ 4                   ; Data sent to the timer output  when tms
				  ; is set hight.

;*****************************************************************************
;*                             PROGRAM SPACE                                 *
;*****************************************************************************

;|===========================================================================|
;|                             main program                                  |
;|===========================================================================|

	 .org 880h

reset	 ldi wdr,watchtim         ; Load the watchdog.
	 ldi ior,10h              ; Enables all interrupts.
	 reti  

	 ldi tcr,count		  ; 12 x 16 x (0fah) x 1/8 Mhz = 6 ms
	 ldi tscr,01101100b       ; Start the timer. 

loop     wait                     ; Wait for interrupt.
	 jp loop                  ; Continue.

;|===========================================================================|
;|                          interrupt routine                                |
;|===========================================================================|

;|---------------------------------------------------------------------------|
;|                            timer interrupt                                |
;|---------------------------------------------------------------------------|

it_timer ldi wdr,watchtim          ; Reload the watchdog.
	 ldi tcr,count             ; Reload the counter.
	 res tmz,tscr              ; Restart the timer.
	 jrr dout,tscr,lo_hi       ; Check status of dout.
				   ; If low, set high next time.
hi_lo    res dout,tscr             ; Clr dout next time.
	 jp end_tim
lo_hi    set dout,tscr             ; Set dout next time.
end_tim  reti

;*****************************************************************************
;*                     Restart and interrupt Vectors                         *
;*****************************************************************************

	 .org 0ff0h

adc      nop
	 reti
timer    jp it_timer
int2     nop
         reti
int1     nop
	 reti
	 nop
	 nop
	 nop
	 nop
nmi      nop
	 reti
res      jp reset
